@@ -124,8 +124,8 @@ char *dStrstrip(char *s)
    int len;
 
    if (s && *s) {
-      for (p = s; isspace(*p); ++p);
-      for (len = strlen(p); len && isspace(p[len-1]); --len);
+      for (p = s; dIsspace(*p); ++p);
+      for (len = strlen(p); len && dIsspace(p[len-1]); --len);
       if (p > s)
          memmove(s, p, len);
       s[len] = 0;
@@ -133,6 +133,15 @@ char *dStrstrip(char *s)
    return s;
 }
 
+/*
+ * Clear the contents of the string
+ */
+void dStrshred(char *s)
+{
+   if (s)
+      memset(s, 0, strlen(s));
+}
+
 /*
  * Return a new string of length 'len' filled with 'c' characters
  */
@@ -163,26 +172,56 @@ char *dStrsep(char **orig, const char *delim)
    return str;
 }
 
+/*
+ * ASCII functions to avoid the case difficulties introduced by I/i in
+ * Turkic locales.
+ */
+
 /*
  * Case insensitive strstr
  */
-char *dStristr(const char *haystack, const char *needle)
+char *dStriAsciiStr(const char *haystack, const char *needle)
 {
    int i, j;
+   char *ret = NULL;
 
-   for (i = 0, j = 0; haystack[i] && needle[j]; ++i)
-      if (tolower(haystack[i]) == tolower(needle[j])) {
-         ++j;
-      } else if (j) {
-         i -= j;
-         j = 0;
-      }
-
-   if (!needle[j])                 /* Got all */
-      return (char *)(haystack + i - j);
-   return NULL;
+   if (haystack && needle) {
+      for (i = 0, j = 0; haystack[i] && needle[j]; ++i)
+         if (D_ASCII_TOLOWER(haystack[i]) == D_ASCII_TOLOWER(needle[j])) {
+            ++j;
+         } else if (j) {
+            i -= j;
+            j = 0;
+         }
+      if (!needle[j])                 /* Got all */
+         ret = (char *)(haystack + i - j);
+   }
+   return ret;
 }
 
+int dStrAsciiCasecmp(const char *s1, const char *s2)
+{
+   int ret = 0;
+
+   while ((*s1 || *s2) &&
+          !(ret = D_ASCII_TOLOWER(*s1) - D_ASCII_TOLOWER(*s2))) {
+      s1++;
+      s2++;
+   }
+   return ret;
+}
+
+int dStrnAsciiCasecmp(const char *s1, const char *s2, size_t n)
+{
+   int ret = 0;
+
+   while (n-- && (*s1 || *s2) &&
+          !(ret = D_ASCII_TOLOWER(*s1) - D_ASCII_TOLOWER(*s2))) {
+      s1++;
+      s2++;
+   }
+   return ret;
+}
 
 /*
  *- dStr ----------------------------------------------------------------------
@@ -333,6 +372,15 @@ void dStr_truncate (Dstr *ds, int len)
    }
 }
 
+/*
+ * Clear a Dstr.
+ */
+void dStr_shred (Dstr *ds)
+{
+   if (ds && ds->sz > 0)
+      memset(ds->str, '\0', ds->sz);
+}
+
 /*
  * Erase a substring.
  */
@@ -785,17 +833,17 @@ int dParser_parse_rc_line(char **line, char **name, char **value)
       ret = 1;
    } else if ((eq = strchr(*line, '='))) {
       /* get name */
-      for (p = *line; *p && *p != '=' && !isspace(*p); ++p);
+      for (p = *line; *p && *p != '=' && !dIsspace(*p); ++p);
       *p = 0;
       *name = *line;
 
       /* skip whitespace */
       if (p < eq)
-         for (++p; isspace(*p); ++p);
+         for (++p; dIsspace(*p); ++p);
 
       /* get value */
       if (p == eq) {
-         for (++p; isspace(*p); ++p);
+         for (++p; dIsspace(*p); ++p);
          len = strlen(p);
          if (len >= 2 && *p == '"' && p[len-1] == '"') {
             p[len-1] = 0;
@@ -875,11 +923,6 @@ char *dGetline (FILE *stream)
 
    dstr = dStr_sized_new(64);
    while ((ch = fgetc(stream)) != EOF) {
-      if (ch == '\\') {
-         /* continue with the next line */
-         while ((ch = fgetc(stream)) != EOF && ch != '\n') ;
-         continue;
-      }
       dStr_append_c(dstr, ch);
       if (ch == '\n')
          break;
