@@ -15,6 +15,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
+#include <errno.h>
 
 #include "../dpip/dpip.h"
 #include "dpiutil.h"
@@ -33,9 +35,21 @@
 /*
  * Global variables
  */
-static SockHandler *sh = NULL;
+static Dsh *sh = NULL;
 
+static void b64strip_illegal_chars(unsigned char* str)
+{
+   unsigned char *p, *s = str;
 
+   MSG("len=%d{%s}\n", strlen((char*)str), str);
+
+   for (p = s; (*p = *s); ++s) {
+      if (isascii(*p) && (isalnum(*p) || strchr("+/=", *p)))
+         ++p;
+   }
+
+   MSG("len=%d{%s}\n", strlen((char *)str), str);
+}
 
 static int b64decode(unsigned char* str)
 {
@@ -153,16 +167,16 @@ static void send_decoded_data(const char *url, const char *mime_type,
 
    /* Send dpip tag */
    d_cmd = a_Dpip_build_cmd("cmd=%s url=%s", "start_send_page", url);
-   sock_handler_write_str(sh, 1, d_cmd);
+   a_Dpip_dsh_write_str(sh, 1, d_cmd);
    dFree(d_cmd);
 
    /* Send HTTP header. */
-   sock_handler_write_str(sh, 0, "Content-type: ");
-   sock_handler_write_str(sh, 0, mime_type);
-   sock_handler_write_str(sh, 1, "\n\n");
+   a_Dpip_dsh_write_str(sh, 0, "Content-type: ");
+   a_Dpip_dsh_write_str(sh, 0, mime_type);
+   a_Dpip_dsh_write_str(sh, 1, "\n\n");
 
    /* Send message */
-   sock_handler_write(sh, 0, (char *)data, data_sz);
+   a_Dpip_dsh_write(sh, 0, (char *)data, data_sz);
 }
 
 static void send_failure_message(const char *url, const char *mime_type,
@@ -180,24 +194,24 @@ static void send_failure_message(const char *url, const char *mime_type,
 
    /* Send dpip tag */
    d_cmd = a_Dpip_build_cmd("cmd=%s url=%s", "start_send_page", url);
-   sock_handler_write_str(sh, 1, d_cmd);
+   a_Dpip_dsh_write_str(sh, 1, d_cmd);
    dFree(d_cmd);
 
    /* Send HTTP header. */
-   sock_handler_write_str(sh, 0, "Content-type: ");
-   sock_handler_write_str(sh, 0, msg_mime_type);
-   sock_handler_write_str(sh, 1, "\n\n");
+   a_Dpip_dsh_write_str(sh, 0, "Content-type: ");
+   a_Dpip_dsh_write_str(sh, 0, msg_mime_type);
+   a_Dpip_dsh_write_str(sh, 1, "\n\n");
 
    /* Send message */
-   sock_handler_write_str(sh, 0, msg);
+   a_Dpip_dsh_write_str(sh, 0, msg);
 
    /* send some debug info */
    snprintf(buf, 1024, "mime_type: %s<br>data size: %d<br>data: %s<br>",
             mime_type, (int)data_sz, data);
-   sock_handler_write_str(sh, 0, buf);
+   a_Dpip_dsh_write_str(sh, 0, buf);
 
    /* close page */
-   sock_handler_write_str(sh, 0, "</body></html>");
+   a_Dpip_dsh_write_str(sh, 0, "</body></html>");
 }
 
 /*
@@ -212,14 +226,14 @@ static char *datauri_get_mime(char *url)
    char *mime_type = NULL, *p;
    size_t len = 0;
 
-   if (dStrncasecmp(url, "data:", 5) == 0) {
+   if (dStrnAsciiCasecmp(url, "data:", 5) == 0) {
       if ((p = strchr(url, ',')) && p - url < 256) {
          url += 5;
          len = p - url;
          strncpy(buf, url, len);
          buf[len] = 0;
          /* strip ";base64" */
-         if (len >= 7 && dStrcasecmp(buf + len - 7, ";base64") == 0) {
+         if (len >= 7 && dStrAsciiCasecmp(buf + len - 7, ";base64") == 0) {
             len -= 7;
             buf[len] = 0;
          }
@@ -228,7 +242,7 @@ static char *datauri_get_mime(char *url)
       /* that's it, now handle omitted types */
       if (len == 0) {
          mime_type = dStrdup("text/plain;charset=US-ASCII");
-      } else if (!dStrncasecmp(buf, "charset", 7)) {
+      } else if (!dStrnAsciiCasecmp(buf, "charset", 7)) {
          mime_type = dStrconcat("text/plain", buf, NULL);
       } else {
          mime_type = dStrdup(buf);
@@ -248,14 +262,15 @@ static unsigned char *datauri_get_data(char *url, size_t *p_sz)
    unsigned char *data = NULL;
 
    if ((p = strchr(url, ',')) && p - url >= 12 &&  /* "data:;base64" */
-       dStrncasecmp(p - 7, ";base64", 7) == 0) {
+       dStrnAsciiCasecmp(p - 7, ";base64", 7) == 0) {
       is_base64 = 1;
    }
 
    if (p) {
       ++p;
       if (is_base64) {
-         data = (unsigned char *)dStrdup(p);
+         data = (unsigned char *)Unescape_uri_str(p);
+         b64strip_illegal_chars(data);
          *p_sz = (size_t) b64decode(data);
       } else {
          data = (unsigned char *)a_Url_decode_hex_str(p, p_sz);
@@ -279,7 +294,7 @@ int main(void)
    size_t data_size = 0;
 
    /* Initialize the SockHandler */
-   sh = sock_handler_new(STDIN_FILENO, STDOUT_FILENO, 8*1024);
+   sh = a_Dpip_dsh_new(STDIN_FILENO, STDOUT_FILENO, 8*1024);
 
    rc = chdir("/tmp");
    if (rc == -1) {
@@ -287,12 +302,21 @@ int main(void)
           dStrerror(errno));
    }
 
+   /* Authenticate our client... */
+   if (!(dpip_tag = a_Dpip_dsh_read_token(sh, 1)) ||
+       a_Dpip_check_auth(dpip_tag) < 0) {
+      MSG("can't authenticate request: %s\n", dStrerror(errno));
+      a_Dpip_dsh_close(sh);
+      return 1;
+   }
+   dFree(dpip_tag);
+
    /* Read the dpi command from STDIN */
-   dpip_tag = sock_handler_read(sh);
+   dpip_tag = a_Dpip_dsh_read_token(sh, 1);
    MSG("[%s]\n", dpip_tag);
 
-   cmd = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "cmd");
-   url = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "url");
+   cmd = a_Dpip_get_attr(dpip_tag, "cmd");
+   url = a_Dpip_get_attr(dpip_tag, "url");
    if (!cmd || !url) {
       MSG("Error, cmd=%s, url=%s\n", cmd, url);
       exit (EXIT_FAILURE);
@@ -321,8 +345,8 @@ int main(void)
    dFree(dpip_tag);
 
    /* Finish the SockHandler */
-   sock_handler_close(sh);
-   sock_handler_free(sh);
+   a_Dpip_dsh_close(sh);
+   a_Dpip_dsh_free(sh);
 
    return 0;
 }
