@@ -23,17 +23,14 @@
 #include "../lout/msg.h"
 #include "../lout/misc.hh"
 
-#include <fltk/draw.h>
-#include <fltk/Color.h>
+#include <FL/fl_draw.H>
 
 #define IMAGE_MAX_AREA (6000 * 6000)
 
-using namespace fltk;
-
 namespace dw {
 namespace fltk {
 
-using namespace container::typed;
+using namespace lout::container::typed;
 
 FltkImgbuf::FltkImgbuf (Type type, int width, int height)
 {
@@ -67,11 +64,11 @@ void FltkImgbuf::init (Type type, int width, int height, FltkImgbuf *root)
 
    refCount = 1;
    deleteOnUnref = true;
-   copiedRows = new misc::BitSet (height);
+   copiedRows = new lout::misc::BitSet (height);
 
    // The list is only used for root buffers.
    if (isRoot())
-      scaledBuffers = new container::typed::List <FltkImgbuf> (true);
+      scaledBuffers = new lout::container::typed::List <FltkImgbuf> (true);
    else
       scaledBuffers = NULL;
 
@@ -193,7 +190,7 @@ core::Imgbuf* FltkImgbuf::getScaledBuf (int width, int height)
     */
    if (width <= 0 || height <= 0 ||
        width > IMAGE_MAX_AREA / height) {
-      MSG("FltkImgbuf::getScaledBuf: suspicious image size request %dx%d\n",
+      MSG("FltkImgbuf::getScaledBuf: suspicious image size request %d x %d\n",
            width, height);
       ref ();
       return this;
@@ -302,7 +299,7 @@ int FltkImgbuf::scaledY(int ySrc)
    return ySrc * height / root->height;
 }
 
-void FltkImgbuf::draw (::fltk::Widget *target, int xRoot, int yRoot,
+void FltkImgbuf::draw (Fl_Widget *target, int xRoot, int yRoot,
                        int x, int y, int width, int height)
 {
    // TODO: Clarify the question, whether "target" is the current widget
@@ -324,10 +321,9 @@ void FltkImgbuf::draw (::fltk::Widget *target, int xRoot, int yRoot,
       height = this->height - y;
    }
 
-   // Draw
-   ::fltk::Rectangle rect (xRoot + x, yRoot + y, width, height);
-   PixelType ptype = (type == RGBA) ? ::fltk::RGBA : ::fltk::RGB;
-   drawimage(rawdata+bpp*(y*this->width + x),ptype,rect,bpp*this->width);
+   fl_draw_image(rawdata+bpp*(y*this->width + x), xRoot + x, yRoot + y, width,
+                 height, bpp, this->width * bpp);
+
 }
 
 } // namespace dw
