@@ -141,7 +141,7 @@ FltkFont::~FltkFont ()
 static void strstrip(char *big, const char *little)
 {
    if (strlen(big) >= strlen(little) &&
-      strcasecmp(big + strlen(big) - strlen(little), little) == 0)
+      misc::AsciiStrcasecmp(big + strlen(big) - strlen(little), little) == 0)
       *(big + strlen(big) - strlen(little)) = '\0';
 }
 
@@ -232,10 +232,14 @@ FltkColor::FltkColor (int color): Color (color)
 {
    this->color = color;
 
-   colors[SHADING_NORMAL] = shadeColor (color, SHADING_NORMAL) << 8;
-   colors[SHADING_INVERSE] = shadeColor (color, SHADING_INVERSE) << 8;
-   colors[SHADING_DARK] = shadeColor (color, SHADING_DARK) << 8;
-   colors[SHADING_LIGHT] = shadeColor (color, SHADING_LIGHT) << 8;
+   if (!(colors[SHADING_NORMAL] = shadeColor (color, SHADING_NORMAL) << 8))
+      colors[SHADING_NORMAL] = FL_BLACK;
+   if (!(colors[SHADING_INVERSE] = shadeColor (color, SHADING_INVERSE) << 8))
+      colors[SHADING_INVERSE] = FL_BLACK;
+   if (!(colors[SHADING_DARK] = shadeColor (color, SHADING_DARK) << 8))
+      colors[SHADING_DARK] = FL_BLACK;
+   if (!(colors[SHADING_LIGHT] = shadeColor (color, SHADING_LIGHT) << 8))
+      colors[SHADING_LIGHT] = FL_BLACK;
 }
 
 FltkColor::~FltkColor ()
@@ -301,6 +305,7 @@ void FltkTooltip::onEnter()
       Fl_Box *b = new Fl_Box(0,0,100,24);
       b->box(FL_BORDER_BOX);
       b->color(fl_color_cube(FL_NUM_RED-1, FL_NUM_GREEN-1, FL_NUM_BLUE-2));
+      b->labelcolor(FL_BLACK);
       b->labelfont(FL_HELVETICA);
       b->labelsize(14);
       b->align(FL_ALIGN_WRAP|FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
@@ -520,14 +525,18 @@ int FltkPlatform::textWidth (core::style::Font *font, const char *text,
          if ((cu = fl_toupper(c)) == c) {
             /* already uppercase, just draw the character */
             fl_font(ff->font, ff->size);
-            width += font->letterSpacing;
-            width += (int)fl_width(text + curr, next - curr);
+            if (fl_nonspacing(cu) == 0) {
+               width += font->letterSpacing;
+               width += (int)fl_width(text + curr, next - curr);
+            }
          } else {
             /* make utf8 string for converted char */
             nb = fl_utf8encode(cu, chbuf);
             fl_font(ff->font, sc_fontsize);
-            width += font->letterSpacing;
-            width += (int)fl_width(chbuf, nb);
+            if (fl_nonspacing(cu) == 0) {
+               width += font->letterSpacing;
+               width += (int)fl_width(chbuf, nb);
+            }
          }
       }
    } else {
@@ -539,7 +548,9 @@ int FltkPlatform::textWidth (core::style::Font *font, const char *text,
 
          while (next < len) {
             next = nextGlyph(text, curr);
-            width += font->letterSpacing;
+            c = fl_utf8decode(text + curr, text + next, &nb);
+            if (fl_nonspacing(c) == 0)
+               width += font->letterSpacing;
             curr = next;
          }
       }
@@ -548,6 +559,36 @@ int FltkPlatform::textWidth (core::style::Font *font, const char *text,
    return width;
 }
 
+char *FltkPlatform::textToUpper (const char *text, int len)
+{
+   char *newstr = NULL;
+
+   if (len > 0) {
+      int newlen;
+
+      newstr = (char*) malloc(3 * len + 1);
+      newlen = fl_utf_toupper((const unsigned char*)text, len, newstr);
+      assert(newlen <= 3 * len);
+      newstr[newlen] = '\0';
+   }
+   return newstr;
+}
+
+char *FltkPlatform::textToLower (const char *text, int len)
+{
+   char *newstr = NULL;
+
+   if (len > 0) {
+      int newlen;
+
+      newstr = (char*) malloc(3 * len + 1);
+      newlen = fl_utf_tolower((const unsigned char*)text, len, newstr);
+      assert(newlen <= 3 * len);
+      newstr[newlen] = '\0';
+   }
+   return newstr;
+}
+
 int FltkPlatform::nextGlyph (const char *text, int idx)
 {
    return fl_utf8fwd (&text[idx + 1], text, &text[strlen (text)]) - text;
