@@ -25,6 +25,7 @@
 
 #define MAX misc::max
 
+using namespace lout;
 
 namespace dw {
 
@@ -33,7 +34,9 @@ int Table::CLASS_ID = -1;
 Table::Table(bool limitTextWidth)
 {
    registerName ("dw::Table", &CLASS_ID);
+   setFlags (BLOCK_LEVEL);
    setFlags (USES_HINTS);
+   setButtonSensitive(false);
 
    this->limitTextWidth = limitTextWidth;
 
@@ -267,23 +270,24 @@ core::Iterator *Table::iterator (core::Content::Type mask, bool atEnd)
 
 void Table::addCell (Widget *widget, int colspan, int rowspan)
 {
+   const int maxspan = 100;
    Child *child;
    int colspanEff;
 
-   // We limit the values for colspan and rowspan to 50, to avoid
+   // We limit the values for colspan and rowspan to avoid
    // attacks by malicious web pages.
-   if (colspan > 50 || colspan < 0) {
-      MSG_WARN("colspan = %d is set to 50.\n", colspan);
-      colspan = 50;
+   if (colspan > maxspan || colspan < 0) {
+      MSG_WARN("colspan = %d is set to %d.\n", colspan, maxspan);
+      colspan = maxspan;
    }
-   if (rowspan > 50 || rowspan <= 0) {
-      MSG_WARN("rowspan = %d is set to 50.\n", rowspan);
-      rowspan = 50;
+   if (rowspan > maxspan || rowspan <= 0) {
+      MSG_WARN("rowspan = %d is set to %d.\n", rowspan, maxspan);
+      rowspan = maxspan;
    }
 
    if (numRows == 0) {
       // to prevent a crash
-      MSG_WARN("Cell without row.\n");
+      MSG("addCell: cell without row.\n");
       addRow (NULL);
    }
 
@@ -317,11 +321,19 @@ void Table::addCell (Widget *widget, int colspan, int rowspan)
    for (int col = 0; col < colspanEff; col++)
       for (int row = 0; row < rowspan; row++)
          if (!(col == 0 && row == 0)) {
+            int i = (curRow + row) * numCols + curCol + col;
+
+            child = children->get(i);
+            if (child) {
+               MSG("Overlapping spans in table.\n");
+               assert(child->type == Child::SPAN_SPACE);
+               delete child;
+            }
             child = new Child ();
             child->type = Child::SPAN_SPACE;
             child->spanSpace.startCol = curCol;
             child->spanSpace.startRow = curRow;
-            children->set ((curRow + row) * numCols + curCol + col, child);
+            children->set (i, child);
          }
 
    // Set the "root" cell.
