@@ -125,11 +125,6 @@ const char *const AboutSplash=
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
 "    <td>\n"
-"     <a href='http://www.mathaba.net/'>Mathaba</a>\n"
-"    <tr>\n"
-"    <tr>\n"
-"    <td>&nbsp;&nbsp;\n"
-"    <td>\n"
 "     <a href='http://www.commondreams.org/'>C.&nbsp;Dreams</a>\n"
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
@@ -161,7 +156,7 @@ const char *const AboutSplash=
 "    <td><a href='http://www.gutenberg.org/'>P.&nbsp;Gutenberg</a>\n"
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
-"    <td><a href='http://freshmeat.net/'>FreshMeat</a>\n"
+"    <td><a href='http://freecode.com/'>Freecode</a>\n"
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
 "    <td><a href='http://www.gnu.org/gnu/thegnuproject.html'>GNU\n"
@@ -216,8 +211,8 @@ const char *const AboutSplash=
 " <td bgcolor='#FFFFFF'>\n"
 "  <table border='0' cellspacing='0' cellpadding='5'><tr><td>\n"
 "  <p>\n"
-"  Dillo is Free Software under the terms of version 3 of the\n"
-"  <a href='http://www.gnu.org/licenses/gpl.html'>GPL</a>.\n"
+"  The Dillo web browser is Free Software under the terms of version 3 of\n"
+"  the <a href='http://www.gnu.org/licenses/gpl.html'>GPL</a>.\n"
 "  This means you have four basic freedoms:\n"
 "  <ul>\n"
 "   <li>Freedom to use the program any way you see fit.\n"
@@ -239,25 +234,24 @@ const char *const AboutSplash=
 "<tr>\n"
 " <td bgcolor='#CCCCCC'>\n"
 "  <h4>Release overview</h4>\n"
-"  September 06, 2011\n"
+"  December 05, 2011\n"
 "<tr>\n"
 " <td bgcolor='#FFFFFF'>\n"
 "  <table border='0' cellspacing='0' cellpadding='5'>\n"
 "  <tr>\n"
 "   <td>\n"
 "<p>\n"
-"Dillo-3.0 is a port to FLTK-1.3, which is big news because FLTK-1.3.0 was\n"
-"<a href='http://fltk.org/articles.php?L1086'>released</a>\n"
-"in June, clearing the way for Dillo to return to those distributions\n"
-"which had excluded Dillo2 due to FLTK2 never being officially released.\n"
+"dillo-3.0.2 brings you some new bits and pieces, as listed below :)\n"
 "<p>\n"
-"Dillo-3.0 also has plenty of improvements and bugfixes.\n"
-"<p>\n"
-"After this release, the core team will focus on implementing the CSS\n"
+"After this release, the core team plans to focus on implementing the CSS\n"
 "feature of floating elements. This will <em>greatly</em> improve dillo's\n"
 "web page rendering since many sites have adopted floats instead of tables.\n"
 "<p>\n"
-"The new dillo3 has shown excellent stability in our experience.\n"
+"Dillo3 uses the FLTK GUI toolkit's 1.3.x series. In June, fltk-1.3.0 was\n"
+"<a href='http://fltk.org/articles.php?L1086'>released</a>,\n"
+"clearing the way for Dillo to return to those distributions\n"
+"which had excluded Dillo2 due to FLTK2 never being officially released.\n"
+"<p>\n"
 "The core team welcomes developers willing to join our workforce.\n"
 "<p>\n"
 "  </table>\n"
@@ -280,39 +274,11 @@ const char *const AboutSplash=
 "  <tr>\n"
 "   <td>\n"
 "<ul>\n"
-"<li>Ported Dillo to FLTK-1.3.\n"
-"<li>Native build on OSX.\n"
-"<li>Default binding for close-all changed from Alt-q to Ctrl-q.\n"
-"<li>Default binding for close-tab changed from Ctrl-q to Ctrl-w.\n"
-"<li>Default binding for left-tab changed to Shift-Ctrl-Tab.\n"
-"<li>Rewrote the User Interface: much simpler design and event handling.\n"
-"<li>Added on-the-fly panel resize (tiny/small/medium and normal/small icons).\n"
-"<li>'hide-panels' key action now hides the findbar if present,\n"
-"     and toggles display of the control panels otherwise.\n"
-"<li>Allow multiple search engines to be set in dillorc, with a menu\n"
-"    in the web search dialog to select between them.\n"
-"<li>Added an optional label to dillorc's search_url.\n"
-"    Format: \"[&lt;label&gt; ]&lt;url&gt;\"\n"
-"<li>Add right_click_closes_tab preference (default is middle click).\n"
-"<li>Allow binding to non-ASCII keys and multimedia keys.\n"
-"<li>Avoid a certificate dialog storm on some HTTPS sites (BUG#868).\n"
-"<li>Enable line wrapping for &lt;textarea&gt;. (BUG#903)\n"
-"<li>Avoid double render after going Back or Forward\n"
-"    (it takes half the time now!)\n"
-"<li>Implemented a custom tabs handler (to allow fine control of it).\n"
-"<li>Rewrote dw's crossing-events dispatcher (avoids redundant events).\n"
-"<li>Fixed a years old bug: stamped tooltips when scrolling with keyboard.\n"
-"<li>Fixed a border case in URL resolver: empty path + {query|fragment}\n"
-"     (BUG#948)\n"
-"<li>Cancel the expected URL after offering a download (BUG#982)\n"
-"<li>Eliminated a pack of 22 compiler warnings (gcc-4.6.1 amd64)\n"
-"<li>Removed 'large' option of panel_size preference.\n"
-"<li>Removed --enable-ansi configure option.\n"
-"<li>Limit saved cookie size.\n"
-"<li>Wrap image alt text.\n"
-"<li>Added support for CSS adjacent sibling selectors.\n"
-"<li>Fix redraw loops and reenabled limit_text_width dillorc option.\n"
-"<li>Collapse parent's and first child's top margin.\n"
+"<li>Digest authentication.\n"
+"<li>Rework line breaking and fix white-space:nowrap handling.\n"
+"<li>text-transform property.\n"
+"<li>Locale-independent ASCII character case handling (fixes Turkic locales).\n"
+"<li>Bind Ctrl-{PageUp,PageDown} to tab-{previous,next}.\n"
 "</ul>\n"
 "  </table>\n"
 "</table>\n"
@@ -333,12 +299,12 @@ const char *const AboutSplash=
 "<ul>\n"
 " <li> There's a\n"
 "   <a href='http://www.dillo.org/dillorc'>dillorc</a>\n"
-"   (readable  config)  file within the tarball; It is well-commented\n"
+"   (readable  config)  file inside the tarball. It is well-commented\n"
 "   and  has  plenty  of  options to customize dillo, so <STRONG>copy\n"
 "   it</STRONG>  to  your  <STRONG>~/.dillo/</STRONG>  directory, and\n"
 "   modify it to your taste.\n"
 " <li> Documentation for developers is in the <CODE>/doc</CODE>\n"
-"   dir  within  the  tarball;  you can find directions on everything\n"
+"   dir  inside  the  tarball;  you can find directions on everything\n"
 "   else at the home page.\n"
 " <li> The right mouse button brings up a context-sensitive menu\n"
 "   (available on pages, links, images, forms, the Back and Forward buttons,\n"
