@@ -67,7 +67,7 @@ static Viewer_t Mime_minor_type_fetch(const char *Key, uint_t Size)
 
    if (Size) {
       for ( i = 0; i < MimeMinItemsSize; ++i )
-         if (dStrncasecmp(Key, MimeMinItems[i].Name, Size) == 0)
+         if (dStrnAsciiCasecmp(Key, MimeMinItems[i].Name, Size) == 0)
             return MimeMinItems[i].Data;
    }
    return NULL;
@@ -83,7 +83,7 @@ static Viewer_t Mime_major_type_fetch(const char *Key, uint_t Size)
 
    if (Size) {
       for ( i = 0; i < MimeMajItemsSize; ++i )
-         if (dStrncasecmp(Key, MimeMajItems[i].Name, Size) == 0)
+         if (dStrnAsciiCasecmp(Key, MimeMajItems[i].Name, Size) == 0)
             return MimeMajItems[i].Data;
    }
    return NULL;
@@ -108,6 +108,7 @@ void a_Mime_init()
    Mime_add_minor_type("image/x-png", a_Dicache_png_image);    /* deprecated */
 #endif
    Mime_add_minor_type("text/html", a_Html_text);
+   Mime_add_minor_type("application/xhtml+xml", a_Html_text);
 
    /* Add a major type to handle all the text stuff */
    Mime_add_major_type("text", a_Plain_text);
