@@ -47,19 +47,31 @@ typedef struct {
    const char *const *enum_symbols;
 } CssPropertyInfo;
 
+static const char *const Css_border_collapse_enum_vals[] = {
+   "separate", "collapse", NULL
+};
+
+static const char *const Css_border_color_enum_vals[] = {
+   "transparent", NULL
+};
+
 static const char *const Css_border_style_enum_vals[] = {
    "none", "hidden", "dotted", "dashed", "solid", "double", "groove",
    "ridge", "inset", "outset", NULL
 };
 
-static const char *const Css_cursor_enum_vals[] = {
-   "crosshair", "default", "pointer", "move", "e_resize", "ne_resize",
-   "nw_resize", "n_resize", "se_resize", "sw_resize", "s_resize",
-   "w_resize", "text", "wait", "help", NULL
+static const char *const Css_border_width_enum_vals[] = {
+   "thin", "medium", "thick", NULL
 };
 
-static const char *const Css_display_enum_vals[DISPLAY_LAST + 1] = {
-   "block", "inline", "list-item", "table", "table-row-group",
+static const char *const Css_cursor_enum_vals[] = {
+   "crosshair", "default", "pointer", "move", "e-resize", "ne-resize",
+   "nw-resize", "n-resize", "se-resize", "sw-resize", "s-resize",
+   "w-resize", "text", "wait", "help", NULL
+};
+
+static const char *const Css_display_enum_vals[] = {
+   "block", "inline", "list-item", "none", "table", "table-row-group",
    "table-header-group", "table-footer-group", "table-row",
    "table-cell", NULL
 };
@@ -73,10 +85,26 @@ static const char *const Css_font_style_enum_vals[] = {
    "normal", "italic", "oblique", NULL
 };
 
+static const char *const Css_font_variant_enum_vals[] = {
+   "normal", "small-caps", NULL
+};
+
 static const char *const Css_font_weight_enum_vals[] = {
    "bold", "bolder", "light", "lighter", "normal", NULL
 };
 
+static const char *const Css_letter_spacing_enum_vals[] = {
+   "normal", NULL
+};
+
+static const char *const Css_list_style_position_enum_vals[] = {
+   "inside", "outside", NULL
+};
+
+static const char *const Css_line_height_enum_vals[] = {
+   "normal", NULL
+};
+
 static const char *const Css_list_style_type_enum_vals[] = {
    "disc", "circle", "square", "decimal", "decimal-leading-zero",
    "lower-roman", "upper-roman", "lower-greek", "lower-alpha",
@@ -93,12 +121,21 @@ static const char *const Css_text_decoration_enum_vals[] = {
    "underline", "overline", "line-through", "blink", NULL
 };
 
+static const char *const Css_text_transform_enum_vals[] = {
+   "none", "capitalize", "uppercase", "lowercase", NULL
+};
+
 static const char *const Css_vertical_align_vals[] = {
-   "top", "bottom", "middle", "baseline", "sub", "super", NULL
+   "top", "bottom", "middle", "baseline", "sub", "super", "text-top",
+   "text-bottom", NULL
 };
 
 static const char *const Css_white_space_vals[] = {
-   "normal", "pre", "nowrap", NULL
+   "normal", "pre", "nowrap", "pre-wrap", "pre-line", NULL
+};
+
+static const char *const Css_word_spacing_enum_vals[] = {
+   "normal", NULL
 };
 
 const CssPropertyInfo Css_property_info[CSS_PROPERTY_LAST] = {
@@ -107,24 +144,33 @@ const CssPropertyInfo Css_property_info[CSS_PROPERTY_LAST] = {
    {"background-image", {CSS_TYPE_UNUSED}, NULL},
    {"background-position", {CSS_TYPE_UNUSED}, NULL},
    {"background-repeat", {CSS_TYPE_UNUSED}, NULL},
-   {"border-bottom-color", {CSS_TYPE_COLOR, CSS_TYPE_UNUSED}, NULL},
+   {"border-bottom-color", {CSS_TYPE_ENUM, CSS_TYPE_COLOR, CSS_TYPE_UNUSED},
+    Css_border_color_enum_vals},
    {"border-bottom-style", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
     Css_border_style_enum_vals},
-   {"border-bottom-width", {CSS_TYPE_LENGTH, CSS_TYPE_UNUSED}, NULL},
-   {"border-collapse", {CSS_TYPE_UNUSED}, NULL},
-   {"border-left-color", {CSS_TYPE_COLOR, CSS_TYPE_UNUSED}, NULL},
+   {"border-bottom-width", {CSS_TYPE_ENUM, CSS_TYPE_LENGTH, CSS_TYPE_UNUSED},
+    Css_border_width_enum_vals},
+   {"border-collapse", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
+    Css_border_collapse_enum_vals},
+   {"border-left-color", {CSS_TYPE_ENUM, CSS_TYPE_COLOR, CSS_TYPE_UNUSED},
+    Css_border_color_enum_vals},
    {"border-left-style", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
     Css_border_style_enum_vals},
-   {"border-left-width", {CSS_TYPE_LENGTH, CSS_TYPE_UNUSED}, NULL},
-   {"border-right-color", {CSS_TYPE_COLOR, CSS_TYPE_UNUSED}, NULL},
+   {"border-left-width", {CSS_TYPE_ENUM, CSS_TYPE_LENGTH, CSS_TYPE_UNUSED},
+    Css_border_width_enum_vals},
+   {"border-right-color", {CSS_TYPE_ENUM, CSS_TYPE_COLOR, CSS_TYPE_UNUSED},
+    Css_border_color_enum_vals},
    {"border-right-style", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
     Css_border_style_enum_vals},
-   {"border-right-width", {CSS_TYPE_LENGTH, CSS_TYPE_UNUSED}, NULL},
+   {"border-rigth-width", {CSS_TYPE_ENUM, CSS_TYPE_LENGTH, CSS_TYPE_UNUSED},
+    Css_border_width_enum_vals},
    {"border-spacing", {CSS_TYPE_LENGTH, CSS_TYPE_UNUSED}, NULL},
-   {"border-top-color", {CSS_TYPE_COLOR, CSS_TYPE_UNUSED}, NULL},
+   {"border-top-color", {CSS_TYPE_ENUM, CSS_TYPE_COLOR, CSS_TYPE_UNUSED},
+    Css_border_color_enum_vals},
    {"border-top-style", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
     Css_border_style_enum_vals},
-   {"border-top-width", {CSS_TYPE_LENGTH, CSS_TYPE_UNUSED}, NULL},
+   {"border-top-width", {CSS_TYPE_ENUM, CSS_TYPE_LENGTH, CSS_TYPE_UNUSED},
+    Css_border_width_enum_vals},
    {"bottom", {CSS_TYPE_UNUSED}, NULL},
    {"caption-side", {CSS_TYPE_UNUSED}, NULL},
    {"clear", {CSS_TYPE_UNUSED}, NULL},
@@ -144,15 +190,20 @@ const CssPropertyInfo Css_property_info[CSS_PROPERTY_LAST] = {
    {"font-size-adjust", {CSS_TYPE_UNUSED}, NULL},
    {"font-stretch", {CSS_TYPE_UNUSED}, NULL},
    {"font-style", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED}, Css_font_style_enum_vals},
-   {"font-variant", {CSS_TYPE_UNUSED}, NULL},
+   {"font-variant", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
+    Css_font_variant_enum_vals},
    {"font-weight", {CSS_TYPE_ENUM, CSS_TYPE_FONT_WEIGHT, CSS_TYPE_UNUSED},
     Css_font_weight_enum_vals},
    {"height", {CSS_TYPE_LENGTH_PERCENTAGE, CSS_TYPE_UNUSED}, NULL},
    {"left", {CSS_TYPE_UNUSED}, NULL},
-   {"letter-spacing", {CSS_TYPE_UNUSED}, NULL},
-   {"line-height", {CSS_TYPE_UNUSED}, NULL},
+   {"letter-spacing", {CSS_TYPE_ENUM, CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_UNUSED},
+    Css_letter_spacing_enum_vals},
+   {"line-height",
+    {CSS_TYPE_ENUM, CSS_TYPE_LENGTH_PERCENTAGE_NUMBER, CSS_TYPE_UNUSED},
+    Css_line_height_enum_vals},
    {"list-style-image", {CSS_TYPE_UNUSED}, NULL},
-   {"list-style-position", {CSS_TYPE_UNUSED}, NULL},
+   {"list-style-position", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
+    Css_list_style_position_enum_vals},
    {"list-style-type", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
     Css_list_style_type_enum_vals},
    {"margin-bottom", {CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_UNUSED}, NULL},
@@ -179,16 +230,18 @@ const CssPropertyInfo Css_property_info[CSS_PROPERTY_LAST] = {
    {"text-align", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED}, Css_text_align_enum_vals},
    {"text-decoration", {CSS_TYPE_MULTI_ENUM, CSS_TYPE_UNUSED},
     Css_text_decoration_enum_vals},
-   {"text-indent", {CSS_TYPE_UNUSED}, NULL},
+   {"text-indent", {CSS_TYPE_LENGTH_PERCENTAGE, CSS_TYPE_UNUSED}, NULL},
    {"text-shadow", {CSS_TYPE_UNUSED}, NULL},
-   {"text-transform", {CSS_TYPE_UNUSED}, NULL},
+   {"text-transform", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
+    Css_text_transform_enum_vals},
    {"top", {CSS_TYPE_UNUSED}, NULL},
    {"unicode-bidi", {CSS_TYPE_UNUSED}, NULL},
    {"vertical-align",{CSS_TYPE_ENUM, CSS_TYPE_UNUSED},Css_vertical_align_vals},
    {"visibility", {CSS_TYPE_UNUSED}, NULL},
    {"white-space", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED}, Css_white_space_vals},
    {"width", {CSS_TYPE_LENGTH_PERCENTAGE, CSS_TYPE_UNUSED}, NULL},
-   {"word-spacing", {CSS_TYPE_UNUSED}, NULL},
+   {"word-spacing", {CSS_TYPE_ENUM, CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_UNUSED},
+    Css_word_spacing_enum_vals},
    {"z-index", {CSS_TYPE_UNUSED}, NULL},
 
    /* These are extensions, for internal used, and never parsed. */
@@ -405,6 +458,28 @@ void CssParser::ungetChar()
    bufptr--;
 }
 
+/*
+ * Skip string str if it is found in the input buffer.
+ * If string is found leave bufptr pointing to last matched char.
+ * If not wind back. The first char is passed as parameter c
+ * to avoid unnecessary getChar() / ungetChar() calls.
+ */
+inline bool CssParser::skipString(int c, const char *str)
+{
+   for (int n = 0; str[n]; n++) {
+      if (n > 0)
+         c = getChar();
+
+      if (str[n] != c) {
+         while (n--)
+            ungetChar();
+         return false;
+      }
+   }
+
+   return true;
+}
+
 void CssParser::nextToken()
 {
    int c, c1, d, j;
@@ -414,26 +489,16 @@ void CssParser::nextToken()
    ttype = CSS_TK_CHAR; /* init */
    spaceSeparated = false;
 
-   c = getChar();
-
    while (true) {
-      if (isspace(c)) { // ignore whitespace
+      c = getChar();
+      if (isspace(c)) {                    // ignore whitespace
          spaceSeparated = true;
-         c = getChar();
-      } else if (c == '/') { // ignore comments
-         d = getChar();
-         if (d == '*') {
+      } else if (skipString(c, "/*")) {    // ignore comments
+         do {
             c = getChar();
-            d = getChar();
-            while (d != EOF && (c != '*' || d != '/')) {
-               c = d;
-               d = getChar();
-            }
-            c = getChar();
-         } else {
-            ungetChar();
-            break;
-         }
+         } while (c != EOF && ! skipString(c, "*/"));
+      } else if (skipString(c, "<!--")) {  // ignore XML comment markers
+      } else if (skipString(c, "-->")) {
       } else {
          break;
       }
@@ -596,9 +661,10 @@ void CssParser::nextToken()
 }
 
 
-bool CssParser::tokenMatchesProperty(CssPropertyName prop, CssValueType * type)
+bool CssParser::tokenMatchesProperty(CssPropertyName prop, CssValueType *type)
 {
    int i, err = 1;
+   CssValueType savedType = *type;
 
    for (int j = 0; Css_property_info[prop].type[j] != CSS_TYPE_UNUSED; j++) {
       *type = Css_property_info[prop].type[j];
@@ -608,7 +674,7 @@ bool CssParser::tokenMatchesProperty(CssPropertyName prop, CssValueType * type)
       case CSS_TYPE_ENUM:
          if (ttype == CSS_TK_SYMBOL) {
             for (i = 0; Css_property_info[prop].enum_symbols[i]; i++)
-               if (dStrcasecmp(tval,
+               if (dStrAsciiCasecmp(tval,
                      Css_property_info[prop].enum_symbols[i]) == 0)
                   return true;
          }
@@ -616,11 +682,11 @@ bool CssParser::tokenMatchesProperty(CssPropertyName prop, CssValueType * type)
 
       case CSS_TYPE_MULTI_ENUM:
          if (ttype == CSS_TK_SYMBOL) {
-            if (dStrcasecmp(tval, "none") == 0)
-                  return true;
-            else {
+            if (dStrAsciiCasecmp(tval, "none") == 0) {
+               return true;
+            } else {
                for (i = 0; Css_property_info[prop].enum_symbols[i]; i++) {
-                  if (dStrcasecmp(tval,
+                  if (dStrAsciiCasecmp(tval,
                         Css_property_info[prop].enum_symbols[i]) == 0)
                      return true;
                }
@@ -629,6 +695,7 @@ bool CssParser::tokenMatchesProperty(CssPropertyName prop, CssValueType * type)
          break;
 
       case CSS_TYPE_LENGTH_PERCENTAGE:
+      case CSS_TYPE_LENGTH_PERCENTAGE_NUMBER:
       case CSS_TYPE_LENGTH:
          if  (tval[0] == '-')
             return false;
@@ -636,14 +703,14 @@ bool CssParser::tokenMatchesProperty(CssPropertyName prop, CssValueType * type)
       case CSS_TYPE_SIGNED_LENGTH:
          if (ttype == CSS_TK_DECINT ||
              ttype == CSS_TK_FLOAT ||
-             (ttype == CSS_TK_SYMBOL && dStrcasecmp(tval, "auto") == 0))
+             (ttype == CSS_TK_SYMBOL && dStrAsciiCasecmp(tval, "auto") == 0))
             return true;
          break;
 
       case CSS_TYPE_COLOR:
          if ((ttype == CSS_TK_COLOR ||
               ttype == CSS_TK_SYMBOL) &&
-            (dStrcasecmp(tval, "rgb") == 0 ||
+            (dStrAsciiCasecmp(tval, "rgb") == 0 ||
              a_Color_parse(tval, -1, &err) != -1))
             return true;
          break;
@@ -676,7 +743,7 @@ bool CssParser::tokenMatchesProperty(CssPropertyName prop, CssValueType * type)
       }
    }
 
-   *type = CSS_TYPE_UNUSED;
+   *type = savedType;
    return false;
 }
 
@@ -685,9 +752,9 @@ bool CssParser::parseRgbColorComponent(int32_t *cc, int *percentage) {
       MSG_CSS("expected integer not found in %s color\n", "rgb");
       return false;
    }
-   
+
    *cc = strtol(tval, NULL, 10);
-   
+
    nextToken();
    if (ttype == CSS_TK_CHAR && tval[0] == '%') {
       if (*percentage == 0) {
@@ -765,12 +832,13 @@ bool CssParser::parseValue(CssPropertyName prop,
    bool found, ret = false;
    float fval;
    int i, ival, err = 1;
+   Dstr *dstr;
 
    switch (type) {
    case CSS_TYPE_ENUM:
       if (ttype == CSS_TK_SYMBOL) {
          for (i = 0; Css_property_info[prop].enum_symbols[i]; i++)
-            if (dStrcasecmp(tval,
+            if (dStrAsciiCasecmp(tval,
                             Css_property_info[prop].enum_symbols[i]) == 0) {
                val->intVal = i;
                ret = true;
@@ -785,10 +853,10 @@ bool CssParser::parseValue(CssPropertyName prop,
       ret = true;
 
       while (ttype == CSS_TK_SYMBOL) {
-         if (dStrcasecmp(tval, "none") != 0) {
+         if (dStrAsciiCasecmp(tval, "none") != 0) {
             for (i = 0, found = false;
                  !found && Css_property_info[prop].enum_symbols[i]; i++) {
-               if (dStrcasecmp(tval,
+               if (dStrAsciiCasecmp(tval,
                                Css_property_info[prop].enum_symbols[i]) == 0)
                   val->intVal |= (1 << i);
             }
@@ -798,56 +866,68 @@ bool CssParser::parseValue(CssPropertyName prop,
       break;
 
    case CSS_TYPE_LENGTH_PERCENTAGE:
+   case CSS_TYPE_LENGTH_PERCENTAGE_NUMBER:
    case CSS_TYPE_LENGTH:
    case CSS_TYPE_SIGNED_LENGTH:
       if (ttype == CSS_TK_DECINT || ttype == CSS_TK_FLOAT) {
          fval = atof(tval);
-         lentype = CSS_LENGTH_TYPE_PX;  /* Actually, there must be a unit,
-                                         * except for num == 0. */
-
-         ret = true;
+         lentype = CSS_LENGTH_TYPE_NONE;
 
          nextToken();
-         if (ttype == CSS_TK_SYMBOL) {
-            if (dStrcasecmp(tval, "px") == 0) {
+         if (!spaceSeparated && ttype == CSS_TK_SYMBOL) {
+            ret = true;
+
+            if (dStrAsciiCasecmp(tval, "px") == 0) {
                lentype = CSS_LENGTH_TYPE_PX;
                nextToken();
-            } else if (dStrcasecmp(tval, "mm") == 0) {
+            } else if (dStrAsciiCasecmp(tval, "mm") == 0) {
                lentype = CSS_LENGTH_TYPE_MM;
                nextToken();
-            } else if (dStrcasecmp(tval, "cm") == 0) {
+            } else if (dStrAsciiCasecmp(tval, "cm") == 0) {
                lentype = CSS_LENGTH_TYPE_MM;
                fval *= 10;
                nextToken();
-            } else if (dStrcasecmp(tval, "in") == 0) {
+            } else if (dStrAsciiCasecmp(tval, "in") == 0) {
                lentype = CSS_LENGTH_TYPE_MM;
                fval *= 25.4;
                nextToken();
-            } else if (dStrcasecmp(tval, "pt") == 0) {
+            } else if (dStrAsciiCasecmp(tval, "pt") == 0) {
                lentype = CSS_LENGTH_TYPE_MM;
                fval *= (25.4 / 72);
                nextToken();
-            } else if (dStrcasecmp(tval, "pc") == 0) {
+            } else if (dStrAsciiCasecmp(tval, "pc") == 0) {
                lentype = CSS_LENGTH_TYPE_MM;
                fval *= (25.4 / 6);
                nextToken();
-            } else if (dStrcasecmp(tval, "em") == 0) {
+            } else if (dStrAsciiCasecmp(tval, "em") == 0) {
                lentype = CSS_LENGTH_TYPE_EM;
                nextToken();
-            } else if (dStrcasecmp(tval, "ex") == 0) {
+            } else if (dStrAsciiCasecmp(tval, "ex") == 0) {
                lentype = CSS_LENGTH_TYPE_EX;
                nextToken();
+            } else {
+               ret = false;
             }
-         } else if (type == CSS_TYPE_LENGTH_PERCENTAGE &&
+         } else if (!spaceSeparated &&
+                    (type == CSS_TYPE_LENGTH_PERCENTAGE ||
+                     type == CSS_TYPE_LENGTH_PERCENTAGE_NUMBER) &&
                     ttype == CSS_TK_CHAR &&
                     tval[0] == '%') {
             fval /= 100;
             lentype = CSS_LENGTH_TYPE_PERCENTAGE;
+            ret = true;
             nextToken();
          }
 
+         /* Allow numbers without unit only for 0 or
+          * CSS_TYPE_LENGTH_PERCENTAGE_NUMBER
+          */
+         if (lentype == CSS_LENGTH_TYPE_NONE &&
+            (type == CSS_TYPE_LENGTH_PERCENTAGE_NUMBER || fval == 0.0))
+            ret = true;
+
          val->intVal = CSS_CREATE_LENGTH(fval, lentype);
-      } else if (ttype == CSS_TK_SYMBOL && dStrcasecmp(tval, "auto") == 0) {
+      } else if (ttype == CSS_TK_SYMBOL && !dStrAsciiCasecmp(tval, "auto")) {
          ret = true;
          val->intVal = CSS_LENGTH_TYPE_AUTO;
          nextToken();
@@ -863,7 +943,7 @@ bool CssParser::parseValue(CssPropertyName prop,
             ret = true;
          nextToken();
       } else if (ttype == CSS_TK_SYMBOL) {
-         if (dStrcasecmp(tval, "rgb") == 0) {
+         if (dStrAsciiCasecmp(tval, "rgb") == 0) {
             nextToken();
             if (parseRgbColor(&val->intVal))
                ret = true;
@@ -889,11 +969,23 @@ bool CssParser::parseValue(CssPropertyName prop,
       break;
 
    case CSS_TYPE_SYMBOL:
-      if (ttype == CSS_TK_SYMBOL || ttype == CSS_TK_STRING) {
-         val->strVal = dStrdup(tval);
+      /* Read comma separated list of font family names */
+      dstr = dStr_new("");
+      while (ttype == CSS_TK_SYMBOL || ttype == CSS_TK_STRING ||
+             (ttype == CSS_TK_CHAR && tval[0] == ',')) {
+         if (spaceSeparated)
+            dStr_append_c(dstr, ' ');
+         dStr_append(dstr, tval);
          ret = true;
          nextToken();
       }
+
+      if (ret) {
+         val->strVal = dStrstrip(dstr->str);
+         dStr_free(dstr, 0);
+      } else {
+         dStr_free(dstr, 1);
+      }
       break;
 
    case CSS_TYPE_FONT_WEIGHT:
@@ -930,7 +1022,7 @@ bool CssParser::parseWeight()
    if (ttype == CSS_TK_CHAR && tval[0] == '!') {
       nextToken();
       if (ttype == CSS_TK_SYMBOL &&
-          dStrcasecmp(tval, "important") == 0) {
+          dStrAsciiCasecmp(tval, "important") == 0) {
          nextToken();
          return true;
       }
@@ -944,7 +1036,7 @@ bool CssParser::parseWeight()
  */
 static int Css_property_info_cmp(const void *a, const void *b)
 {
-   return dStrcasecmp(((CssPropertyInfo *) a)->symbol,
+   return dStrAsciiCasecmp(((CssPropertyInfo *) a)->symbol,
                       ((CssPropertyInfo *) b)->symbol);
 }
 
@@ -954,7 +1046,7 @@ static int Css_property_info_cmp(const void *a, const void *b)
  */
 static int Css_shorthand_info_cmp(const void *a, const void *b)
 {
-   return dStrcasecmp(((CssShorthandInfo *) a)->symbol,
+   return dStrAsciiCasecmp(((CssShorthandInfo *) a)->symbol,
                       ((CssShorthandInfo *) b)->symbol);
 }
 
@@ -962,11 +1054,12 @@ void CssParser::parseDeclaration(CssPropertyList * props,
                                  CssPropertyList * importantProps)
 {
    CssPropertyInfo pi = {NULL, {CSS_TYPE_UNUSED}, NULL}, *pip;
-   CssShorthandInfo si, *sip;
+   CssShorthandInfo *sip;
    CssValueType type = CSS_TYPE_UNUSED;
 
    CssPropertyName prop;
    CssPropertyValue val, dir_vals[4];
+   CssValueType  dir_types[4];
    bool found, weight;
    int sh_index, i, j, n;
    int dir_set[4][4] = {
@@ -999,7 +1092,6 @@ void CssParser::parseDeclaration(CssPropertyList * props,
          }
       } else {
          /* Try shorthands. */
-         si.symbol = tval;
          sip =
              (CssShorthandInfo *) bsearch(&pi, Css_shorthand_info,
                                           CSS_SHORTHAND_NUM,
@@ -1052,6 +1144,7 @@ void CssParser::parseDeclaration(CssPropertyList * props,
                          parseValue(Css_shorthand_info[sh_index]
                                     .properties[0], type, &val)) {
                         dir_vals[n] = val;
+                        dir_types[n] = type;
                         n++;
                      } else
                         break;
@@ -1063,13 +1156,12 @@ void CssParser::parseDeclaration(CssPropertyList * props,
                         if (weight && importantProps)
                            importantProps->set(Css_shorthand_info[sh_index]
                                                .properties[i],
-                                               type,
-                                               dir_vals[dir_set[n - 1]
-                                                        [i]]);
+                                               dir_types[dir_set[n - 1][i]],
+                                               dir_vals[dir_set[n - 1][i]]);
                         else
                            props->set(Css_shorthand_info[sh_index]
                                       .properties[i],
-                                      type,
+                                      dir_types[dir_set[n - 1][i]],
                                       dir_vals[dir_set[n - 1][i]]);
                   } else
                      MSG_CSS("no values for shorthand property '%s'\n",
@@ -1118,15 +1210,15 @@ void CssParser::parseDeclaration(CssPropertyList * props,
 
 bool CssParser::parseSimpleSelector(CssSimpleSelector *selector)
 {
-   char **pp;
+   CssSimpleSelector::SelectType selectType;
 
    if (ttype == CSS_TK_SYMBOL) {
-      selector->element = a_Html_tag_index(tval);
+      selector->setElement (a_Html_tag_index(tval));
       nextToken();
       if (spaceSeparated)
          return true;
    } else if (ttype == CSS_TK_CHAR && tval[0] == '*') {
-      selector->element = CssSimpleSelector::ELEMENT_ANY;
+      selector->setElement (CssSimpleSelector::ELEMENT_ANY);
       nextToken();
       if (spaceSeparated)
          return true;
@@ -1140,18 +1232,18 @@ bool CssParser::parseSimpleSelector(CssSimpleSelector *selector)
    }
 
    do {
-      pp = NULL;
+      selectType = CssSimpleSelector::SELECT_NONE;
       if (ttype == CSS_TK_CHAR) {
          switch (tval[0]) {
          case '#':
-            pp = &selector->id;
+            selectType = CssSimpleSelector::SELECT_ID;
             break;
          case '.':
-            pp = &selector->klass;
+            selectType = CssSimpleSelector::SELECT_CLASS;
             break;
          case ':':
-            pp = &selector->pseudo;
-            if (*pp)
+            selectType = CssSimpleSelector::SELECT_PSEUDO_CLASS;
+            if (selector->getPseudoClass ())
                // pseudo class has been set already.
                // As dillo currently only supports :link and :visisted, a
                // selector with more than one pseudo class will never match.
@@ -1162,14 +1254,13 @@ bool CssParser::parseSimpleSelector(CssSimpleSelector *selector)
          }
       }
 
-      if (pp) {
+      if (selectType != CssSimpleSelector::SELECT_NONE) {
          nextToken();
          if (spaceSeparated)
             return true;
 
          if (ttype == CSS_TK_SYMBOL) {
-            if (*pp == NULL)
-               *pp = dStrdup(tval);
+            selector->setSelect (selectType, tval);
             nextToken();
          } else {
             return false; // don't accept classes or id's starting with integer
@@ -1177,7 +1268,7 @@ bool CssParser::parseSimpleSelector(CssSimpleSelector *selector)
          if (spaceSeparated)
             return true;
       }
-   } while (pp);
+   } while (selectType != CssSimpleSelector::SELECT_NONE);
 
    DEBUG_MSG(DEBUG_PARSE_LEVEL, "end of simple selector (%s, %s, %s, %d)\n",
       selector->id, selector->klass,
@@ -1203,6 +1294,9 @@ CssSelector *CssParser::parseSelector()
       } else if (ttype == CSS_TK_CHAR && tval[0] == '>') {
          selector->addSimpleSelector (CssSelector::CHILD);
          nextToken();
+      } else if (ttype == CSS_TK_CHAR && tval[0] == '+') {
+         selector->addSimpleSelector (CssSelector::ADJACENT_SIBLING);
+         nextToken();
       } else if (ttype != CSS_TK_END && spaceSeparated) {
          selector->addSimpleSelector (CssSelector::DESCENDANT);
       } else {
@@ -1297,7 +1391,7 @@ char * CssParser::parseUrl()
    Dstr *urlStr = NULL;
 
    if (ttype != CSS_TK_SYMBOL ||
-      dStrcasecmp(tval, "url") != 0)
+      dStrAsciiCasecmp(tval, "url") != 0)
       return NULL;
 
    nextToken();
@@ -1336,34 +1430,99 @@ char * CssParser::parseUrl()
 void CssParser::parseImport(DilloHtml *html, DilloUrl *baseUrl)
 {
    char *urlStr = NULL;
+   bool importSyntaxIsOK = false;
+   bool mediaSyntaxIsOK = true;
+   bool mediaIsSelected = true;
 
-   if (html != NULL &&
-      ttype == CSS_TK_SYMBOL &&
-      dStrcasecmp(tval, "import") == 0) {
-      nextToken();
+   nextToken();
 
-      if (ttype == CSS_TK_SYMBOL &&
-         dStrcasecmp(tval, "url") == 0)
-         urlStr = parseUrl();
-      else if (ttype == CSS_TK_STRING)
-         urlStr = dStrdup (tval);
+   if (ttype == CSS_TK_SYMBOL &&
+       dStrAsciiCasecmp(tval, "url") == 0)
+      urlStr = parseUrl();
+   else if (ttype == CSS_TK_STRING)
+      urlStr = dStrdup (tval);
 
-      /* Skip all tokens until the expected end. */
-      while (!(ttype == CSS_TK_END ||
-            (ttype == CSS_TK_CHAR && (tval[0] == ';'))))
+   nextToken();
+
+   /* parse a comma-separated list of media */
+   if (ttype == CSS_TK_SYMBOL) {
+      mediaSyntaxIsOK = false;
+      mediaIsSelected = false;
+      while (ttype == CSS_TK_SYMBOL) {
+         if (dStrAsciiCasecmp(tval, "all") == 0 ||
+             dStrAsciiCasecmp(tval, "screen") == 0)
+            mediaIsSelected = true;
          nextToken();
+         if (ttype == CSS_TK_CHAR && tval[0] == ',') {
+            nextToken();
+         } else {
+            mediaSyntaxIsOK = true;
+            break;
+         }
+      }
+   }
 
+   if (mediaSyntaxIsOK &&
+       ttype == CSS_TK_CHAR &&
+       tval[0] == ';') {
+      importSyntaxIsOK = true;
       nextToken();
+   } else
+      ignoreStatement();
 
-      if (urlStr) {
+   if (urlStr) {
+      if (importSyntaxIsOK && mediaIsSelected) {
          MSG("CssParser::parseImport(): @import %s\n", urlStr);
          DilloUrl *url = a_Html_url_new (html, urlStr, a_Url_str(baseUrl),
                                          baseUrl ? 1 : 0);
          a_Html_load_stylesheet(html, url);
          a_Url_free(url);
-         dFree (urlStr);
+      }
+      dFree (urlStr);
+   }
+}
+
+void CssParser::parseMedia()
+{
+   bool mediaSyntaxIsOK = false;
+   bool mediaIsSelected = false;
+
+   nextToken();
+
+   /* parse a comma-separated list of media */
+   while (ttype == CSS_TK_SYMBOL) {
+      if (dStrAsciiCasecmp(tval, "all") == 0 ||
+          dStrAsciiCasecmp(tval, "screen") == 0)
+         mediaIsSelected = true;
+      nextToken();
+      if (ttype == CSS_TK_CHAR && tval[0] == ',') {
+         nextToken();
+      } else {
+         mediaSyntaxIsOK = true;
+         break;
       }
    }
+
+   /* check that the syntax is OK so far */
+   if (!(mediaSyntaxIsOK &&
+         ttype == CSS_TK_CHAR &&
+         tval[0] == '{')) {
+      ignoreStatement();
+      return;
+   }
+
+   /* parse/ignore the block as required */
+   if (mediaIsSelected) {
+      nextToken();
+      while (ttype != CSS_TK_END) {
+         parseRuleset();
+         if (ttype == CSS_TK_CHAR && tval[0] == '}') {
+            nextToken();
+            break;
+         }
+      }
+   } else
+      ignoreBlock();
 }
 
 const char * CssParser::propertyNameString(CssPropertyName name)
@@ -1371,37 +1530,83 @@ const char * CssParser::propertyNameString(CssPropertyName name)
    return Css_property_info[name].symbol;
 }
 
+void CssParser::ignoreBlock()
+{
+   int depth = 0;
+
+   while (ttype != CSS_TK_END) {
+      if (ttype == CSS_TK_CHAR) {
+         if (tval[0] == '{') {
+            depth++;
+         } else if (tval[0] == '}') {
+            depth--;
+            if (depth == 0) {
+               nextToken();
+               return;
+            }
+         }
+      }
+      nextToken();
+   }
+}
+
+void CssParser::ignoreStatement()
+{
+   while (ttype != CSS_TK_END) {
+      if (ttype == CSS_TK_CHAR) {
+         if (tval[0] == ';') {
+            nextToken();
+            return;
+         } else if (tval[0] =='{') {
+            ignoreBlock();
+            return;
+         }
+      }
+      nextToken();
+   }
+}
+
 void CssParser::parse(DilloHtml *html, DilloUrl *url, CssContext * context,
                       const char *buf,
                       int buflen, CssOrigin origin)
 {
    CssParser parser (context, origin, buf, buflen);
+   bool importsAreAllowed = true;
 
-   while (parser.ttype == CSS_TK_CHAR && parser.tval[0] == '@') {
-      parser.nextToken();
-      parser.parseImport(html, url);
+   while (parser.ttype != CSS_TK_END) {
+      if (parser.ttype == CSS_TK_CHAR &&
+          parser.tval[0] == '@') {
+         parser.nextToken();
+         if (parser.ttype == CSS_TK_SYMBOL) {
+            if (dStrAsciiCasecmp(parser.tval, "import") == 0 &&
+                html != NULL &&
+                importsAreAllowed) {
+               parser.parseImport(html, url);
+            } else if (dStrAsciiCasecmp(parser.tval, "media") == 0) {
+               parser.parseMedia();
+            } else {
+               parser.ignoreStatement();
+            }
+         } else {
+            parser.ignoreStatement();
+         }
+      } else {
+         importsAreAllowed = false;
+         parser.parseRuleset();
+      }
    }
-
-   while (parser.ttype != CSS_TK_END)
-      parser.parseRuleset();
 }
 
-CssPropertyList *CssParser::parseDeclarationBlock(const char *buf, int buflen)
+void CssParser::parseDeclarationBlock(const char *buf, int buflen,
+                                      CssPropertyList *props,
+                                      CssPropertyList *propsImortant)
 {
-   CssPropertyList *props = new CssPropertyList (true);
    CssParser parser (NULL, CSS_ORIGIN_AUTHOR, buf, buflen);
 
    parser.withinBlock = true;
 
    do
-      parser.parseDeclaration(props, NULL);
+      parser.parseDeclaration(props, propsImortant);
    while (!(parser.ttype == CSS_TK_END ||
          (parser.ttype == CSS_TK_CHAR && parser.tval[0] == '}')));
-
-   if (props->size () == 0) {
-      delete props;
-      props = NULL;
-   }
-
-   return props;
 }
