@@ -107,7 +107,7 @@ static DICacheEntry *Dicache_entry_new(void)
  * Add a new entry in the dicache
  * (a single node (URL) may have several entries)
  */
-DICacheEntry *a_Dicache_add_entry(const DilloUrl *Url)
+static DICacheEntry *Dicache_add_entry(const DilloUrl *Url)
 {
    DICacheEntry *entry;
    DICacheNode *node;
@@ -325,7 +325,7 @@ void a_Dicache_new_scan(const DilloUrl *url, int version)
    dReturn_if_fail ( DicEntry != NULL );
    if (DicEntry->State < DIC_SetParms) {
       MSG("a_Dicache_new_scan before DIC_SetParms\n");
-      exit(-1);
+      exit(1);
    }
    a_Bitvec_clear(DicEntry->BitVec);
    DicEntry->ScanNumber++;
@@ -409,7 +409,7 @@ static void *Dicache_image(int ImgType, const char *MimeType, void *Ptr,
    DicEntry = a_Dicache_get_entry(web->url, DIC_Last);
    if (!DicEntry) {
       /* Let's create an entry for this image... */
-      DicEntry = a_Dicache_add_entry(web->url);
+      DicEntry = Dicache_add_entry(web->url);
       DicEntry->DecoderData =
          (ImgType == DIC_Png) ?
             a_Png_new(web->Image, DicEntry->url, DicEntry->version) :
@@ -573,7 +573,7 @@ void a_Dicache_freeall(void)
          a_Imgbuf_unref(entry->v_imgbuf);
          dicache_size_total -= entry->TotalSize;
       }
-      dList_remove(CachedIMGs, node);
+      dList_remove_fast(CachedIMGs, node);
       a_Url_free(node->url);
       dFree(node);
    }
