@@ -8,13 +8,17 @@ extern "C" {
 #endif /* __cplusplus */
 
 
-typedef void (*DnsCallback_t)(int Status, Dlist *addr_list, void *data);
+typedef void (*DnsCallback_t)(int status, Dlist *addr_list, void *data);
 
 void a_Dns_init (void);
 void a_Dns_freeall(void);
 void a_Dns_resolve(const char *hostname, DnsCallback_t cb_func, void *cb_data);
 
-#define DILLO_ADDR_MAX sizeof(struct in6_addr)
+#ifdef ENABLE_IPV6
+#  define DILLO_ADDR_MAX sizeof(struct in6_addr)
+#else
+#  define DILLO_ADDR_MAX sizeof(struct in_addr)
+#endif
 
 typedef struct _DilloHost
 {
