@@ -351,8 +351,6 @@ static size_t Gif_do_extension(DilloGif *gif, uint_t Label,
       return Gif_data_blocks(buf, BSize);
 
    case Txt_Ext:                /* Plain text Extension */
-      /* This extension allows (rcm thinks) the image to be rendered as text.
-       */
    case App_Ext:                /* Application Extension */
    default:
       return Gif_do_generic_ext(buf, BSize);    /*Ignore Extension */
@@ -427,16 +425,6 @@ static void Gif_emit_line(DilloGif *gif, const uchar_t *linebuf)
    }
 }
 
-/*
- * I apologize for the large size of this routine and the goto error
- * construct - I almost _never_ do that. I offer the excuse of
- * optimizing for speed.
- *
- * RCM -- busted these down into smaller subroutines... still very hard to
- * read.
- */
-
-
 /*
  * Decode the packetized lwz bytes
  */
@@ -689,7 +677,7 @@ static int Gif_decode(DilloGif *gif, const uchar_t *buf, size_t bsize)
             case 2:         /* End code... consume remaining data chunks..? */
                goto error;  /* Could clean up better? */
             default:
-               printf("dillo_gif_decode: error!\n");
+               MSG("Gif_decode: error!\n");
                goto error;
             }
          }
@@ -816,14 +804,12 @@ static size_t Gif_do_img_desc(DilloGif *gif, void *Buf,
    /* check max image size */
    if (gif->Width <= 0 || gif->Height <= 0 ||
        gif->Width > IMAGE_MAX_AREA / gif->Height) {
-      MSG("Gif_do_img_desc: suspicious image size request %ux%u\n",
+      MSG("Gif_do_img_desc: suspicious image size request %u x %u\n",
           gif->Width, gif->Height);
       gif->state = 999;
       return 0;
    }
 
-   gif->linebuf = dMalloc(gif->Width);
-
    a_Dicache_set_parms(gif->url, gif->version, gif->Image,
                        gif->Width, gif->Height, DILLO_IMG_TYPE_INDEXED);
 
@@ -854,6 +840,7 @@ static size_t Gif_do_img_desc(DilloGif *gif, void *Buf,
    gif->y = 0;
    Gif_lwz_init(gif);
    gif->spill_line_index = 0;
+   gif->linebuf = dMalloc(gif->Width);
    gif->state = 3;              /*Process the lzw data next */
    if (gif->Image && gif->ColorMap_ofs) {
       a_Dicache_set_cmap(gif->url, gif->version, gif->Image,
