@@ -819,11 +819,11 @@ static const char *const tools_xpm[] = {
 "                      ",
 "                      ",
 "        .XXo.         ",
-"        oO $$+.       ",
+"        oO$$$+.       ",
 "        oo@$$$@#      ",
 "         #o@$$$O#     ",
 "          #X$$$$%X    ",
-"  &#%      #X$$$@o    ",
+"  &#       #X$$$@o    ",
 "  #*o      =-@$$$#    ",
 "  oO*o    #$@$$$$#    ",
 "  oOOOX  oO@;$$$$X    ",
@@ -1426,11 +1426,11 @@ static const char *const tools_s_xpm[] = {
 "     p*45<Xg    ",
 "     h.>885oa   ",
 "      p.3883Xg  ",
-" i$O   s$488;s  ",
+"  $O   s$488;s  ",
 " O>1p  e;588;a  ",
 " O331sr;7778:s  ",
 " O343,**3678;a  ",
-" f,643;#%368:s  ",
+" O,643;#%368:s  ",
 "  t<64>= &374f  ",
 "   t244>= &379ih",
 "    d@:;=& &475u",
@@ -1520,6 +1520,42 @@ static const char *const search_xpm[] = {
 "         2}2  ",
 "         11   "};
 
+/* XPM */
+static const char *const help_xpm[] = {
+"14 16 16 1",
+"       c None",
+"1      c #DBDBDB",
+"2      c #B6B6B6",
+"3      c #929292",
+"4      c #6D6D6D",
+"5      c #F1EFEF",
+"6      c #018B00",
+"7      c #A48A53",
+"8      c #A4BDA4",
+"9      c #000000",
+"A      c #000000",
+"B      c #000000",
+"C      c #000000",
+"D      c #000000",
+"E      c #000000",
+"F      c #000000",
+"              ",
+"    444333    ",
+"   4   4384   ",
+"  44    4283  ",
+"  4444  4824  ",
+"  4444  4274  ",
+"   44  4274   ",
+"       474    ",
+"      474     ",
+"      44      ",
+"      44      ",
+"              ",
+"      44      ",
+"     4764     ",
+"     4674     ",
+"      44      "};
+
 /* XPM */
 static const char *const full_screen_on_xpm[] = {
 "13 15 2 1",
