@@ -35,12 +35,15 @@ using namespace dw::core;
 
 class DilloPlain {
 private:
-   class PlainEventReceiver: public dw::core::Widget::EventReceiver {
+   class PlainLinkReceiver: public dw::core::Layout::LinkReceiver {
    public:
       DilloPlain *plain;
-      bool buttonPress(dw::core::Widget *widget, dw::core::EventButton *event);
+      bool press(dw::core::Widget *widget, int link, int img, int x, int y,
+                 dw::core::EventButton *event);
    };
-   PlainEventReceiver plainReceiver;
+   PlainLinkReceiver plainReceiver;
+
+   void addLine(char *Buf, uint_t BufSize);
 
 public:
    BrowserWindow *bw;
@@ -91,7 +94,8 @@ DilloPlain::DilloPlain(BrowserWindow *p_bw)
    Start_Ofs = 0;
    state = ST_SeekingEol;
 
-   StyleEngine styleEngine ((Layout*)bw->render_layout);
+   Layout *layout = (Layout*) bw->render_layout;
+   StyleEngine styleEngine (layout);
 
    styleEngine.startElement ("body");
    styleEngine.startElement ("pre");
@@ -99,7 +103,7 @@ DilloPlain::DilloPlain(BrowserWindow *p_bw)
    widgetStyle->ref ();
 
    /* The context menu */
-   DW2TB(dw)->connectEvent (&plainReceiver);
+   layout->connectLink (&plainReceiver);
 
    /* Hook destructor to the dw delete call */
    dw->setDeleteCallback(a_Plain_free, this);
@@ -117,10 +121,10 @@ DilloPlain::~DilloPlain()
 /*
  * Receive the mouse button press event
  */
-bool DilloPlain::PlainEventReceiver::buttonPress (Widget *widget,
-                                                  EventButton *event)
+bool DilloPlain::PlainLinkReceiver::press (Widget *widget, int, int, int, int,
+                                           EventButton *event)
 {
-   _MSG("DilloPlain::PlainEventReceiver::buttonPress\n");
+   _MSG("DilloPlain::PlainLinkReceiver::buttonPress\n");
 
    if (event->button == 3) {
       a_UIcmd_page_popup(plain->bw, FALSE, NULL);
@@ -129,6 +133,25 @@ bool DilloPlain::PlainEventReceiver::buttonPress (Widget *widget,
    return false;
 }
 
+void DilloPlain::addLine(char *Buf, uint_t BufSize)
+{
+   int len;
+   char buf[128];
+   char *end = Buf + BufSize;
+
+   if (BufSize > 0) {
+      // Limit word length to avoid X11 coordinate
+      // overflow with extremely long lines.
+      while ((len = a_Misc_expand_tabs(&Buf, end, buf, sizeof(buf))))
+         DW2TB(dw)->addText(buf, len, widgetStyle);
+   } else {
+      // Add dummy word for empty lines - otherwise the parbreak is ignored.
+      DW2TB(dw)->addText("", 0, widgetStyle);
+   }
+
+   DW2TB(dw)->addParbreak(0, widgetStyle);
+}
+
 /*
  * Here we parse plain text and put it into the page structure.
  * (This function is called by Plain_callback whenever there's new data)
@@ -136,7 +159,6 @@ bool DilloPlain::PlainEventReceiver::buttonPress (Widget *widget,
 void DilloPlain::write(void *Buf, uint_t BufSize, int Eof)
 {
    char *Start;
-   char *data;
    uint_t i, len, MaxBytes;
 
    _MSG("DilloPlain::write Eof=%d\n", Eof);
@@ -154,10 +176,7 @@ void DilloPlain::write(void *Buf, uint_t BufSize, int Eof)
          }
          break;
       case ST_Eol:
-         data = a_Misc_expand_tabs(Start + i - len, len);
-         DW2TB(dw)->addText(data, widgetStyle);
-         DW2TB(dw)->addParbreak(0, widgetStyle);
-         dFree(data);
+         addLine(Start + i - len, len);
          if (Start[i] == '\r' && Start[i + 1] == '\n') ++i;
          if (i < MaxBytes) ++i;
          state = ST_SeekingEol;
@@ -167,10 +186,7 @@ void DilloPlain::write(void *Buf, uint_t BufSize, int Eof)
    }
    Start_Ofs += i - len;
    if (Eof && len) {
-      data = a_Misc_expand_tabs(Start + i - len, len);
-      DW2TB(dw)->addText(data, widgetStyle);
-      DW2TB(dw)->addParbreak(0, widgetStyle);
-      dFree(data);
+      addLine(Start + i - len, len);
       Start_Ofs += len;
    }
 
