@@ -2,7 +2,6 @@
  * File : url.h - Dillo
  *
  * Copyright (C) 2001 Jorge Arellano Cid <jcid@dillo.org>
- *               2001 Livio Baldini Soares <livio@linux.ime.usp.br>
  *
  * Parse and normalize all URL's inside Dillo.
  */
@@ -48,21 +47,21 @@
  * (non '_'-ended macros MUST use these for initialization sake)
  */
 /* these MAY return NULL: */
-#define URL_SCHEME_(u)              u->scheme
-#define URL_AUTHORITY_(u)           u->authority
-#define URL_PATH_(u)                u->path
-#define URL_QUERY_(u)               u->query
-#define URL_FRAGMENT_(u)            u->fragment
+#define URL_SCHEME_(u)              (u)->scheme
+#define URL_AUTHORITY_(u)           (u)->authority
+#define URL_PATH_(u)                (u)->path
+#define URL_QUERY_(u)               (u)->query
+#define URL_FRAGMENT_(u)            (u)->fragment
 #define URL_HOST_(u)                a_Url_hostname(u)
-#define URL_ALT_(u)                 u->alt
+#define URL_ALT_(u)                 (u)->alt
 #define URL_STR_(u)                 a_Url_str(u)
 /* this returns a Dstr* */
-#define URL_DATA_(u)                u->data
+#define URL_DATA_(u)                (u)->data
 /* these return an integer */
-#define URL_PORT_(u)                (URL_HOST(u), u->port)
-#define URL_FLAGS_(u)               u->flags
-#define URL_ILLEGAL_CHARS_(u)       url->illegal_chars
-#define URL_ILLEGAL_CHARS_SPC_(u)   url->illegal_chars_spc
+#define URL_PORT_(u)                (URL_HOST(u), (u)->port)
+#define URL_FLAGS_(u)               (u)->flags
+#define URL_ILLEGAL_CHARS_(u)       (u)->illegal_chars
+#define URL_ILLEGAL_CHARS_SPC_(u)   (u)->illegal_chars_spc
 
 /*
  * Access methods that never return NULL.
@@ -123,7 +122,7 @@ void a_Url_set_ismap_coords(DilloUrl *u, char *coord_str);
 char *a_Url_decode_hex_str(const char *str);
 char *a_Url_encode_hex_str(const char *str);
 char *a_Url_string_strip_delimiters(const char *str);
-
+bool_t a_Url_same_organization(const DilloUrl *u1, const DilloUrl *u2);
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
