@@ -19,8 +19,8 @@
 
 
 
-#include <fltk/Window.h>
-#include <fltk/run.h>
+#include <FL/Fl_Window.H>
+#include <FL/Fl.H>
 
 #include "../dw/core.hh"
 #include "../dw/fltkcore.hh"
@@ -38,7 +38,8 @@ int main(int argc, char **argv)
    FltkPlatform *platform = new FltkPlatform ();
    Layout *layout = new Layout (platform);
 
-   ::fltk::Window *window = new ::fltk::Window(200, 300, "Dw Border Test");
+   Fl_Window *window = new Fl_Window(200, 300, "Dw Border Test");
+   window->box(FL_NO_BOX);
    window->begin();
 
    FltkViewport *viewport = new FltkViewport (0, 0, 200, 300);
@@ -57,7 +58,9 @@ int main(int argc, char **argv)
    fontAttrs.size = 14;
    fontAttrs.weight = 400;
    fontAttrs.style = FONT_STYLE_NORMAL;
-   styleAttrs.font = Font::create (layout, &fontAttrs);
+   fontAttrs.letterSpacing = 0;
+   fontAttrs.fontVariant = FONT_VARIANT_NORMAL;
+   styleAttrs.font = dw::core::style::Font::create (layout, &fontAttrs);
 
    styleAttrs.color = Color::create (layout, 0x000000);
    styleAttrs.backgroundColor = Color::create (layout, 0xffffff);
@@ -88,7 +91,7 @@ int main(int argc, char **argv)
 
    const char *words1[] = { "Some", "random", "text.", NULL };
    const char *words2[] = { "A", "nested", "paragraph.", NULL };
-   
+
    for(int i = 0; words1[i]; i++) {
       if(i != 0)
          textblock1->addSpace (wordStyle);
@@ -97,7 +100,7 @@ int main(int argc, char **argv)
 
    for(int i = 0; i < 1; i++) {
       textblock1->addParbreak(0, wordStyle);
-      
+
       Textblock *textblock2 = new Textblock (false);
       textblock1->addWidget (textblock2, widgetStyle2);
 
@@ -114,7 +117,7 @@ int main(int argc, char **argv)
 
    window->resizable(viewport);
    window->show();
-   int errorCode = ::fltk::run();
+   int errorCode = Fl::run();
 
    widgetStyle2->unref();
    wordStyle->unref();
