@@ -19,8 +19,9 @@
 
 
 
-#include <fltk/Window.h>
-#include <fltk/run.h>
+#include <FL/Fl.H>
+#include <FL/Fl_Window.H>
+#include <FL/Fl_Box.H>
 
 #include "../dw/core.hh"
 #include "../dw/fltkcore.hh"
@@ -32,35 +33,38 @@ using namespace dw::core;
 using namespace dw::core::style;
 using namespace dw::fltk;
 
-class LinkTestReceiver: public Widget::LinkReceiver
+class LinkTestReceiver: public Layout::LinkReceiver
 {
-   bool enter (Widget *widget, int link, int x, int y);
-   bool press (Widget *widget, int link, int x, int y, EventButton *event);
-   bool release (Widget *widget, int link, int x, int y, EventButton *event);
-   bool click (Widget *widget, int link, int x, int y, EventButton *event);
+   bool enter (Widget *widget, int link, int img, int x, int y);
+   bool press (Widget *widget, int link, int img, int x, int y,
+               EventButton *event);
+   bool release (Widget *widget, int link, int img, int x, int y,
+                 EventButton *event);
+   bool click (Widget *widget, int link, int img, int x, int y,
+               EventButton *event);
 };
 
-bool LinkTestReceiver::enter (Widget *widget, int link, int x, int y)
+bool LinkTestReceiver::enter (Widget *widget, int link, int img, int x, int y)
 {
    printf ("enter: %d\n", link);
    return true;
 }
 
-bool LinkTestReceiver::press (Widget *widget, int link, int x, int y,
+bool LinkTestReceiver::press (Widget *widget, int link, int img, int x, int y,
                               EventButton *event)
 {
    printf ("press: %d\n", link);
    return true;
 }
 
-bool LinkTestReceiver::release (Widget *widget, int link, int x, int y,
+bool LinkTestReceiver::release (Widget *widget, int link, int img, int x,int y,
                                 EventButton *event)
 {
    printf ("release: %d\n", link);
    return true;
 }
 
-bool LinkTestReceiver::click (Widget *widget, int link, int x, int y,
+bool LinkTestReceiver::click (Widget *widget, int link, int img, int x, int y,
                               EventButton *event)
 {
    printf ("click: %d\n", link);
@@ -75,22 +79,23 @@ int main(int argc, char **argv)
    FltkPlatform *platform = new FltkPlatform ();
    Layout *layout = new Layout (platform);
 
-   ::fltk::Window *window = new ::fltk::Window(200, 300, "Dw Links");
+   Fl_Window *window = new Fl_Window(200, 300, "Dw Links2");
+   window->box(FL_NO_BOX);
    window->begin();
-    ::fltk::Widget *Panel = new ::fltk::Widget(0, 0, ww, lh, "CONTROL PANEL");
+    Fl_Widget *Panel = new Fl_Box(0, 0, ww, lh, "CONTROL PANEL");
 
-    Panel->color(::fltk::GRAY15);
-    Panel->labelcolor(::fltk::WHITE);
-    ::fltk::Widget *Main =
-       new ::fltk::Widget(0, lh, ww, wh - 2*lh, "MAIN RENDERING AREA");
-    Main->color(::fltk::GRAY20);
-    Main->labelcolor(::fltk::WHITE);
+    Panel->color(FL_GRAY_RAMP + 3);
+    Panel->labelcolor(FL_WHITE);
+    Panel->box(FL_FLAT_BOX);
+    Fl_Widget *Main = new Fl_Box(0, lh, ww, wh - 2*lh, "MAIN RENDERING AREA");
+    Main->color(FL_GRAY_RAMP + 4);
+    Main->labelcolor(FL_WHITE);
     MainIdx = window->find(Main);
     /* status bar */
-    ::fltk::Widget *Bar =
-       new ::fltk::Widget(0, wh - lh, 200, lh, "STATUS BAR...");
-    Bar->color(::fltk::GRAY15);
-    Bar->labelcolor(::fltk::WHITE);
+    Fl_Widget *Bar = new Fl_Box(0, wh - lh, 200, lh, "STATUS BAR...");
+    Bar->color(FL_GRAY_RAMP + 3);
+    Bar->labelcolor(FL_WHITE);
+    Bar->box(FL_FLAT_BOX);
 
     window->resizable(Main);
    window->end();
@@ -116,7 +121,9 @@ int main(int argc, char **argv)
    fontAttrs.size = 14;
    fontAttrs.weight = 400;
    fontAttrs.style = FONT_STYLE_NORMAL;
-   styleAttrs.font = Font::create (layout, &fontAttrs);
+   fontAttrs.letterSpacing = 0;
+   fontAttrs.fontVariant = FONT_VARIANT_NORMAL;
+   styleAttrs.font = dw::core::style::Font::create (layout, &fontAttrs);
 
    styleAttrs.color = Color::create (layout, 0x000000);
    styleAttrs.backgroundColor = Color::create (layout, 0xffffff);
@@ -127,7 +134,7 @@ int main(int argc, char **argv)
    textblock->setStyle (widgetStyle);
    layout->setWidget (textblock);
 
-   textblock->connectLink (new LinkTestReceiver ());
+   layout->connectLink (new LinkTestReceiver ());
 
    widgetStyle->unref();
 
@@ -140,7 +147,7 @@ int main(int argc, char **argv)
    styleAttrs.color = Color::create (layout, 0x0000ff);
    styleAttrs.textDecoration = TEXT_DECORATION_UNDERLINE;
    styleAttrs.cursor = CURSOR_POINTER;
-   
+
    for(int i = 1; i <= 30; i++) {
       char buf[4];
       sprintf(buf, "%d.", i);
@@ -157,15 +164,15 @@ int main(int argc, char **argv)
          textblock->addText (words1[j], wordStyle);
          textblock->addSpace(wordStyle);
       }
-      
+
       styleAttrs.x_link = i;
       Style *linkStyle = Style::create (layout, &styleAttrs);
-      
+
       for(int j = 0; words2[j]; j++) {
          textblock->addText (words2[j], linkStyle);
          textblock->addSpace(wordStyle);
       }
-      
+
       linkStyle->unref ();
 
       textblock->addParbreak(10, wordStyle);
@@ -177,7 +184,7 @@ int main(int argc, char **argv)
 
    window->resizable(viewport);
    window->show();
-   int errorCode = ::fltk::run();
+   int errorCode = Fl::run();
 
    delete layout;
 
