@@ -19,8 +19,8 @@
 
 
 
-#include <fltk/Window.h>
-#include <fltk/run.h>
+#include <FL/Fl.H>
+#include <FL/Fl_Window.H>
 
 #include "../dw/core.hh"
 #include "../dw/fltkcore.hh"
@@ -38,7 +38,8 @@ int main(int argc, char **argv)
    FltkPlatform *platform = new FltkPlatform ();
    Layout *layout = new Layout (platform);
 
-   ::fltk::Window *window = new ::fltk::Window(200, 300, "Dw Table Aligned");
+   Fl_Window *window = new Fl_Window(200, 300, "Dw Table Aligned");
+   window->box(FL_NO_BOX);
    window->begin();
 
    FltkViewport *viewport = new FltkViewport (0, 0, 200, 300);
@@ -56,7 +57,9 @@ int main(int argc, char **argv)
    fontAttrs.size = 14;
    fontAttrs.weight = 400;
    fontAttrs.style = FONT_STYLE_NORMAL;
-   styleAttrs.font = Font::create (layout, &fontAttrs);
+   fontAttrs.letterSpacing = 0;
+   fontAttrs.fontVariant = FONT_VARIANT_NORMAL;
+   styleAttrs.font = dw::core::style::Font::create (layout, &fontAttrs);
 
    styleAttrs.color = Color::create (layout, 0x000000);
    styleAttrs.backgroundColor = Color::create (layout, 0xa0a0a0);
@@ -73,7 +76,7 @@ int main(int argc, char **argv)
 
    styleAttrs.borderWidth.setVal (1);
    styleAttrs.setBorderStyle (BORDER_INSET);
-  
+
    Style *cellStyle = Style::create (layout, &styleAttrs);
 
    styleAttrs.borderWidth.setVal (0);
@@ -103,14 +106,14 @@ int main(int argc, char **argv)
 
       cell->addText (buf, wordStyle);
       cell->flush ();
-   } 
+   }
 
    wordStyle->unref();
    cellStyle->unref();
 
    window->resizable(viewport);
    window->show();
-   int errorCode = ::fltk::run();
+   int errorCode = Fl::run();
 
    delete layout;
 
